@@ -1165,10 +1165,10 @@ static int mailmbox_expunge_to_file_no_lock(char * dest_filename, int dest_fd,
 {
   int r;
   int res;
-  unsigned long i;
   size_t cur_offset;
   char * dest;
   size_t size;
+  int i;
 
   size = 0;
   for(i = 0 ; i < carray_count(folder->mb_tab) ; i ++) {
